# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
)
from plone.testing import z2

import collective.imagemaps


class CollectiveImagemapsLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.restapi
        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=collective.imagemaps)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.imagemaps:default')


COLLECTIVE_IMAGEMAPS_FIXTURE = CollectiveImagemapsLayer()


COLLECTIVE_IMAGEMAPS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_IMAGEMAPS_FIXTURE,),
    name='CollectiveImagemapsLayer:IntegrationTesting',
)


COLLECTIVE_IMAGEMAPS_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_IMAGEMAPS_FIXTURE,),
    name='CollectiveImagemapsLayer:FunctionalTesting',
)


COLLECTIVE_IMAGEMAPS_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_IMAGEMAPS_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name='CollectiveImagemapsLayer:AcceptanceTesting',
)



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import ViewletBase


class ImageMapViewlet(ViewletBase):
    def update(self):
        self.imagemap = getattr(self.context, "imagemap", "")

    def index(self):
        return self.imagemap or u""



# -*- coding: utf-8 -*-
from collective.imagemaps.behaviors.image_map import IImageMapMarker
from collective.imagemaps.testing import (
    COLLECTIVE_IMAGEMAPS_INTEGRATION_TESTING  # noqa,
)
from plone.app.testing import setRoles, TEST_USER_ID
from plone.behavior.interfaces import IBehavior
from zope.component import getUtility

import unittest


class ImageMapIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_IMAGEMAPS_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_behavior_image_map(self):
        behavior = getUtility(IBehavior, 'collective.imagemaps.image_map')
        self.assertEqual(
            behavior.marker,
            IImageMapMarker,
        )



# -*- coding: utf-8 -*-

from collective.imagemaps import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_hasattr
from zope.component import adapter
from zope.interface import implementer, Interface, provider


class IImageMapMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IImageMap(model.Schema):
    """
    """

    imagemap = schema.Text(
        title=_(u"Image Map Markup",),
        description=_(u"Insert your image map markup here.",),
        default=u"",
        required=False,
        readonly=False,
    )


@implementer(IImageMap)
@adapter(IImageMapMarker)
class ImageMap(object):
    def __init__(self, context):
        self.context = context

    @property
    def imagemap(self):
        if safe_hasattr(self.context, "imagemap"):
            return self.context.imagemap or ""
        return None

    @imagemap.setter
    def imagemap(self, value):
        self.context.imagemap = value



# -*- coding: utf-8 -*-
from collective.imagemaps.interfaces import ICollectiveImagemapsLayer
from collective.imagemaps.testing import (
    COLLECTIVE_IMAGEMAPS_FUNCTIONAL_TESTING,
    COLLECTIVE_IMAGEMAPS_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles, TEST_USER_ID
from Products.Five.browser import BrowserView
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager

import unittest


class ViewletIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_IMAGEMAPS_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.app = self.layer['app']
        self.request = self.app.REQUEST
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(self.portal, 'Document', 'other-document')
        api.content.create(self.portal, 'News Item', 'newsitem')

    def test_image_map_viewlet_is_registered(self):
        view = BrowserView(self.portal['other-document'], self.request)
        manager_name = 'plone.belowcontent'
        alsoProvides(self.request, ICollectiveImagemapsLayer)
        manager = queryMultiAdapter(
            (self.portal['other-document'], self.request, view),
            IViewletManager,
            manager_name,
            default=None
        )
        self.assertIsNotNone(manager)
        manager.update()
        my_viewlet = [v for v in manager.viewlets if v.__name__ == 'image-map-viewlet']  # NOQA: E501
        self.assertEqual(len(my_viewlet), 1)

    # XXX would be nice to have this test working:
    # def test_image_map_viewlet_is_not_available_on_newsitem(self):
    #     view = BrowserView(self.portal['newsitem'], self.request)
    #     manager_name = 'plone.abovecontenttitle'
    #     alsoProvides(self.request, ICollectiveImagemapsLayer)
    #     manager = queryMultiAdapter(
    #         (self.portal['newsitem'], self.request, view),
    #         IViewletManager,
    #         manager_name,
    #         default=None
    #     )
    #     self.assertIsNotNone(manager)
    #     manager.update()
    #     my_viewlet = [v for v in manager.viewlets if v.__name__ == 'image-map-viewlet']  # NOQA: E501
    #     self.assertEqual(len(my_viewlet), 0)


class ViewletFunctionalTest(unittest.TestCase):

    layer = COLLECTIVE_IMAGEMAPS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
